#!/usr/bin/python3
# Test that glibc's sys/mman.h constants match the kernel's.
# Copyright (C) 2018-2025 Free Software Foundation, Inc.
# This file is part of the GNU C Library.
#
# The GNU C Library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# The GNU C Library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with the GNU C Library; if not, see
# <https://www.gnu.org/licenses/>.

import argparse
import sys

import glibcextract
import glibcsyscalls


def main():
    """The main entry point."""
    parser = argparse.ArgumentParser(
        description="Test that glibc's sys/mman.h constants "
        "match the kernel's.")
    parser.add_argument('--cc', metavar='CC',
                        help='C compiler (including options) to use')
    args = parser.parse_args()
    linux_version_headers = glibcsyscalls.linux_kernel_version(args.cc)
    linux_version_glibc = (6, 13)
    sys.exit(glibcextract.compare_macro_consts(
        '#define _GNU_SOURCE 1\n'
        '#include <sys/mman.h>\n',
        '#define _GNU_SOURCE 1\n'
        '#include <linux/mman.h>\n',
        args.cc,
        'MAP_.*',
        # MAP_UNINITIALIZED is defined by the kernel but not by glibc.
        # MAP_FAILED is not a MAP_* flag and is glibc-only, as is the
        # MAP_ANON alias for MAP_ANONYMOUS.  MAP_RENAME, MAP_AUTOGROW,
        # MAP_LOCAL and MAP_AUTORSRV are in the kernel header for
        # MIPS, marked as "not used by linux"; SPARC has MAP_INHERIT
        # in the kernel header, but does not use it.  The kernel
        # header for HPPA removed a define of MAP_VARIABLE to 0 in
        # Linux 6.2.
        'MAP_UNINITIALIZED|MAP_FAILED|MAP_ANON|MAP_RENAME|MAP_AUTOGROW'
        '|MAP_LOCAL|MAP_AUTORSRV|MAP_INHERIT|MAP_VARIABLE',
        linux_version_glibc > linux_version_headers,
        linux_version_headers > linux_version_glibc))

if __name__ == '__main__':
    main()



#!/usr/bin/python3
# Copyright (C) 2023-2025 Free Software Foundation, Inc.
# This file is part of the GNU C Library.
#
# The GNU C Library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# The GNU C Library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with the GNU C Library; if not, see
# <https://www.gnu.org/licenses/>.

import sys

TEMPLATE = """
#include <math.h>
#include <arm_sve.h>

#define STRIDE {stride}
#define PTRUE svptrue_b{prec_short}
#define SV_LOAD svld1_f{prec_short}
#define SV_STORE svst1_f{prec_short}
#define REQUIRE_SVE

#define CALL_BENCH_FUNC_1(v, i) (__extension__ ({{                                       \\
   {rtype} mx0 = {fname}(SV_LOAD (PTRUE(), &variants[v].in->arg0[i * STRIDE]), PTRUE()); \\
   mx0; }}))

#define CALL_BENCH_FUNC_2(v, i) (__extension__ ({{                              \\
   {rtype} mx0 = {fname}(SV_LOAD (PTRUE(), &variants[v].in->arg0[i * STRIDE]),  \\
                         SV_LOAD (PTRUE(), &variants[v].in->arg1[i * STRIDE]),  \\
                         PTRUE());                                              \\
   mx0; }}))

struct args_1
{{
  {stype} arg0[{nelems}];
}};

struct args_2
{{
  {stype} arg0[{nelems}];
  {stype} arg1[{nelems}];
}};

struct _variants
{{
  const char *name;
  const struct args_{arity} *in;
}};

static const struct args_{arity} in0 = {{
{in_data}
}};

static const struct _variants variants[1] = {{
  {{"", &in0}},
}};

#define NUM_VARIANTS 1
#define NUM_SAMPLES(i) ({nelems} / STRIDE)
#define VARIANT(i) (variants[i].name)

// Cannot pass volatile pointer to svst1. This still does not appear to get optimised out.
static {stype} /*volatile*/ ret[{rowlen}];

#define BENCH_FUNC(i, j) ({{ SV_STORE(PTRUE(), ret, CALL_BENCH_FUNC_{arity}(i, j)); }})
#define FUNCNAME "{fname}"
#include <bench-libmvec-skeleton.c>
"""

def main(name):
    _, prec, _, func = name.split("-")
    scalar_to_sve_type = {"double": "svfloat64_t", "float": "svfloat32_t"}
    stride = {"double": "svcntd()", "float": "svcntw()"}[prec]
    rtype = scalar_to_sve_type[prec]
    atype = scalar_to_sve_type[prec]
    prec_short = {"double": 64, "float": 32}[prec]
    # Max SVE vector length is 2048 bits. To ensure benchmarks are
    # vector-length-agnostic, but still use as wide vectors as
    # possible on any given target, divide input data into 2048-bit
    # rows, then load/store as many elements as the target will allow.
    rowlen = {"double": 32, "float": 64}[prec]
    input_filename = {"double": f"{func}-inputs", "float": f"{func}f-inputs"}[prec]

    with open(f"../benchtests/libmvec/{input_filename}") as f:
        input_file = f.readlines()
    in_vals = (l.strip() for l in input_file if l and not l.startswith("#"))
    # Split in case of multivariate signature
    in_vals = (l.split(", ") for l in in_vals)
    # Transpose
    in_vals = list(zip(*in_vals))
    in_data = ",\n".join("{" + (", ".join(val for val in col) + "}")
                         for col in in_vals)

    arity = [l for l in input_file if l.startswith("## args: ")][0].count(prec)
    fname = f"_ZGVsMx{'v' * arity}_{func}{'f' if prec == 'float' else ''}"

    print(TEMPLATE.format(stride=stride,
                          rtype=rtype,
                          atype=atype,
                          fname=fname,
                          prec_short=prec_short,
                          in_data=in_data,
                          stype=prec,
                          rowlen=rowlen,
                          arity=arity,
                          nelems=len(in_vals[0])))


if __name__ == "__main__":
    main(sys.argv[1])



#!/usr/bin/python3
# Print a list of symbols exported by some headers that would
# otherwise be in the user's namespace.
# Copyright (C) 2018-2025 Free Software Foundation, Inc.
# This file is part of the GNU C Library.
#
# The GNU C Library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# The GNU C Library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with the GNU C Library; if not, see
# <https://www.gnu.org/licenses/>.

import argparse

import glibcconform

# Extra symbols possibly not found through -aux-info but still
# reserved by the standard: either data symbols, or symbols where the
# standard leaves unspecified whether the identifier is a macro or
# defined with external linkage.
EXTRA_SYMS = {}
EXTRA_SYMS['ISO'] = {'errno', 'setjmp', 'va_end'}
EXTRA_SYMS['ISO99'] = EXTRA_SYMS['ISO'] | {'math_errhandling'}
# stdatomic.h not yet covered by conformance tests; as per DR#419, all
# the generic functions there or may not be defined with external
# linkage (but are reserved in any case).
EXTRA_SYMS['ISO11'] = EXTRA_SYMS['ISO99']
# The following lists may not be exhaustive.
EXTRA_SYMS['POSIX'] = (EXTRA_SYMS['ISO']
                       | {'environ', 'sigsetjmp', 'optarg', 'optind', 'opterr',
                          'optopt', 'tzname'})
EXTRA_SYMS['XPG4'] = (EXTRA_SYMS['POSIX']
                      | {'signgam', 'loc1', 'loc2', 'locs', 'daylight',
                         'timezone'})
EXTRA_SYMS['XPG42'] = EXTRA_SYMS['XPG4'] | {'getdate_err', 'h_errno'}
EXTRA_SYMS['UNIX98'] = EXTRA_SYMS['XPG42']
EXTRA_SYMS['XOPEN2K'] = (EXTRA_SYMS['POSIX']
                         | {'signgam', 'daylight', 'timezone', 'getdate_err',
                            'h_errno', 'in6addr_any', 'in6addr_loopback'})
EXTRA_SYMS['POSIX2008'] = (EXTRA_SYMS['POSIX']
                           | {'in6addr_any', 'in6addr_loopback'})
EXTRA_SYMS['XOPEN2K8'] = (EXTRA_SYMS['POSIX2008']
                          | {'signgam', 'daylight', 'timezone', 'getdate_err'})


def main():
    """The main entry point."""
    parser = argparse.ArgumentParser(description='List exported symbols.')
    parser.add_argument('--headers', metavar='HEADERS',
                        help='list of headers')
    parser.add_argument('--standard', metavar='STD',
                        help='standard to use when processing headers')
    parser.add_argument('--cc', metavar='CC',
                        help='C compiler to use')
    parser.add_argument('--flags', metavar='CFLAGS',
                        help='Compiler flags to use with CC')
    args = parser.parse_args()
    fns = set()
    compiler = '%s %s' % (args.cc, args.flags)
    for header in args.headers.split():
        fns |= glibcconform.list_exported_functions(compiler, args.standard,
                                                    header)
    fns |= EXTRA_SYMS[args.standard]
    print('\n'.join(sorted(fn for fn in fns if not fn.startswith('_'))))


if __name__ == '__main__':
    main()



#!/usr/bin/python3
# Set Scalable Vector Length test helper
# Copyright (C) 2021-2025 Free Software Foundation, Inc.
# This file is part of the GNU C Library.
#
# The GNU C Library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# The GNU C Library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with the GNU C Library; if not, see
# <https://www.gnu.org/licenses/>.
"""Set Scalable Vector Length test helper.

Set Scalable Vector Length for child process.

examples:

~/build$ make check subdirs=string \
test-wrapper='~/glibc/sysdeps/unix/sysv/linux/aarch64/vltest.py 16'

~/build$ ~/glibc/sysdeps/unix/sysv/linux/aarch64/vltest.py 16 \
make test t=string/test-memcpy

~/build$ ~/glibc/sysdeps/unix/sysv/linux/aarch64/vltest.py 32 \
./debugglibc.sh string/test-memmove

~/build$ ~/glibc/sysdeps/unix/sysv/linux/aarch64/vltest.py 64 \
./testrun.sh string/test-memset
"""
import argparse
from ctypes import cdll, CDLL
import os
import sys

EXIT_SUCCESS = 0
EXIT_FAILURE = 1
EXIT_UNSUPPORTED = 77

AT_HWCAP = 16
HWCAP_SVE = (1 << 22)

PR_SVE_GET_VL = 51
PR_SVE_SET_VL = 50
PR_SVE_SET_VL_ONEXEC = (1 << 18)
PR_SVE_VL_INHERIT = (1 << 17)
PR_SVE_VL_LEN_MASK = 0xffff

def main(args):
    libc = CDLL("libc.so.6")
    if not libc.getauxval(AT_HWCAP) & HWCAP_SVE:
        print("CPU doesn't support SVE")
        sys.exit(EXIT_UNSUPPORTED)

    libc.prctl(PR_SVE_SET_VL,
               args.vl[0] | PR_SVE_SET_VL_ONEXEC | PR_SVE_VL_INHERIT)
    os.execvp(args.args[0], args.args)
    print("exec system call failure")
    sys.exit(EXIT_FAILURE)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description=
            "Set Scalable Vector Length test helper",
            formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    # positional argument
    parser.add_argument("vl", nargs=1, type=int,
                        choices=range(16, 257, 16),
                        help=('vector length '\
                              'which is multiples of 16 from 16 to 256'))
    # remainDer arguments
    parser.add_argument('args', nargs=argparse.REMAINDER,
                        help=('args '\
                              'which is passed to child process'))
    args = parser.parse_args()
    main(args)



#!/usr/bin/python3
# ELF editor for load align tests.
# Copyright (C) 2022-2025 Free Software Foundation, Inc.
# Copyright The GNU Toolchain Authors.
# This file is part of the GNU C Library.
#
# The GNU C Library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# The GNU C Library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with the GNU C Library; if not, see
# <https://www.gnu.org/licenses/>.

import argparse
import os
import sys
import struct

EI_NIDENT=16

EI_MAG0=0
ELFMAG0=b'\x7f'
EI_MAG1=1
ELFMAG1=b'E'
EI_MAG2=2
ELFMAG2=b'L'
EI_MAG3=3
ELFMAG3=b'F'

EI_CLASS=4
ELFCLASSNONE=b'0'
ELFCLASS32=b'\x01'
ELFCLASS64=b'\x02'

EI_DATA=5
ELFDATA2LSB=b'\x01'
ELFDATA2MSB=b'\x02'

ET_EXEC=2
ET_DYN=3

PT_LOAD=1
PT_TLS=7

def elf_types_fmts(e_ident):
    endian = '<' if e_ident[EI_DATA] == ELFDATA2LSB else '>'
    addr = 'I' if e_ident[EI_CLASS] == ELFCLASS32 else 'Q'
    off = 'I' if e_ident[EI_CLASS] == ELFCLASS32 else 'Q'
    return (endian, addr, off)

class Elf_Ehdr:
    def __init__(self, e_ident):
        endian, addr, off = elf_types_fmts(e_ident)
        self.fmt = '{0}HHI{1}{2}{2}IHHHHHH'.format(endian, addr, off)
        self.len = struct.calcsize(self.fmt)

    def read(self, f):
        buf = f.read(self.len)
        if not buf:
            error('{}: header too small'.format(f.name))
        data = struct.unpack(self.fmt, buf)
        self.e_type = data[0]
        self.e_machine = data[1]
        self.e_version = data[2]
        self.e_entry = data[3]
        self.e_phoff = data[4]
        self.e_shoff = data[5]
        self.e_flags = data[6]
        self.e_ehsize = data[7]
        self.e_phentsize= data[8]
        self.e_phnum = data[9]
        self.e_shstrndx = data[10]


class Elf_Phdr:
    def __init__(self, e_ident):
        endian, addr, off = elf_types_fmts(e_ident)
        self.ei_class = e_ident[EI_CLASS]
        if self.ei_class == ELFCLASS32:
            self.fmt = '{0}I{2}{1}{1}IIII'.format(endian, addr, off)
        else:
            self.fmt = '{0}II{2}{1}{1}QQQ'.format(endian, addr, off)
        self.len = struct.calcsize(self.fmt)

    def read(self, f):
        buf = f.read(self.len)
        if len(buf) < self.len:
            error('{}: program header too small'.format(f.name))
        data = struct.unpack(self.fmt, buf)
        if self.ei_class == ELFCLASS32:
            self.p_type = data[0]
            self.p_offset = data[1]
            self.p_vaddr = data[2]
            self.p_paddr = data[3]
            self.p_filesz = data[4]
            self.p_memsz = data[5]
            self.p_flags = data[6]
            self.p_align = data[7]
        else:
            self.p_type = data[0]
            self.p_flags = data[1]
            self.p_offset = data[2]
            self.p_vaddr = data[3]
            self.p_paddr = data[4]
            self.p_filesz = data[5]
            self.p_memsz = data[6]
            self.p_align = data[7]

    def write(self, f):
        if self.ei_class == ELFCLASS32:
            data = struct.pack(self.fmt,
                               self.p_type,
                               self.p_offset,
                               self.p_vaddr,
                               self.p_paddr,
                               self.p_filesz,
                               self.p_memsz,
                               self.p_flags,
                               self.p_align)
        else:
            data = struct.pack(self.fmt,
                               self.p_type,
                               self.p_flags,
                               self.p_offset,
                               self.p_vaddr,
                               self.p_paddr,
                               self.p_filesz,
                               self.p_memsz,
                               self.p_align)
        f.write(data)


def error(msg):
    print(msg, file=sys.stderr)
    sys.exit(1)


def elf_edit_align(phdr, align):
    if align == 'half':
        phdr.p_align = phdr.p_align >> 1
    else:
        phdr.p_align = int(align)

def elf_edit_maximize_tls_size(phdr, elfclass):
    if elfclass == ELFCLASS32:
        # It is possible that the kernel can allocate half of the
        # address space, so use something larger.
        phdr.p_memsz = 0xfff00000
    else:
        phdr.p_memsz = 1 << 63

def elf_edit(f, opts):
    ei_nident_fmt = 'c' * EI_NIDENT
    ei_nident_len = struct.calcsize(ei_nident_fmt)

    data = f.read(ei_nident_len)
    if len(data) < ei_nident_len:
      error('{}: e_nident too small'.format(f.name))
    e_ident = struct.unpack(ei_nident_fmt, data)

    if e_ident[EI_MAG0] != ELFMAG0 \
       or e_ident[EI_MAG1] != ELFMAG1 \
       or e_ident[EI_MAG2] != ELFMAG2 \
       or e_ident[EI_MAG3] != ELFMAG3:
      error('{}: bad ELF header'.format(f.name))

    if e_ident[EI_CLASS] != ELFCLASS32 \
       and e_ident[EI_CLASS] != ELFCLASS64:
      error('{}: unsupported ELF class: {}'.format(f.name, e_ident[EI_CLASS]))

    if e_ident[EI_DATA] != ELFDATA2LSB \
       and e_ident[EI_DATA] != ELFDATA2MSB: \
      error('{}: unsupported ELF data: {}'.format(f.name, e_ident[EI_DATA]))

    ehdr = Elf_Ehdr(e_ident)
    ehdr.read(f)
    if ehdr.e_type not in (ET_EXEC, ET_DYN):
       error('{}: not an executable or shared library'.format(f.name))

    phdr = Elf_Phdr(e_ident)
    maximize_tls_size_done = False
    for i in range(0, ehdr.e_phnum):
        f.seek(ehdr.e_phoff + i * phdr.len)
        phdr.read(f)
        if phdr.p_type == PT_LOAD and opts.align is not None:
            elf_edit_align(phdr, opts.align)
            f.seek(ehdr.e_phoff + i * phdr.len)
            phdr.write(f)
            break
        if phdr.p_type == PT_TLS and opts.maximize_tls_size:
            elf_edit_maximize_tls_size(phdr, e_ident[EI_CLASS])
            f.seek(ehdr.e_phoff + i * phdr.len)
            phdr.write(f)
            maximize_tls_size_done = True
            break

    if opts.maximize_tls_size and not maximize_tls_size_done:
        error('{}: TLS maximum size was not updated'.format(f.name))

def get_parser():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('-a', dest='align',
                        help='How to set the LOAD alignment')
    parser.add_argument('--maximize-tls-size', action='store_true',
                        help='Set maximum PT_TLS size')
    parser.add_argument('output',
                        help='ELF file to edit')
    return parser


def main(argv):
    parser = get_parser()
    opts = parser.parse_args(argv)
    with open(opts.output, 'r+b') as fout:
       elf_edit(fout, opts)


if __name__ == '__main__':
    main(sys.argv[1:])
